@@ -2,7 +2,7 @@
 Module containing all lark internal_transformer classes
 """
 import re
-from typing import Dict, List, Set, Tuple, Union
+from typing import Any, Dict, List, Set, Tuple, Union
 
 import ibis
 from ibis.expr.types import TableExpr
@@ -18,11 +18,11 @@ from sql_to_ibis.parsing.transformers import (
     InternalTransformerWithStarVal,
     TransformerBaseClass,
 )
-from sql_to_ibis.query_info import QueryInfo
-from sql_to_ibis.sql_objects import (
+from sql_to_ibis.query_info import OrderByInfo, QueryInfo
+from sql_to_ibis.sql.sql_clause_objects import LimitExpression, WhereExpression
+from sql_to_ibis.sql.sql_objects import AliasRegistry, AmbiguousColumn
+from sql_to_ibis.sql.sql_value_objects import (
     Aggregate,
-    AliasRegistry,
-    AmbiguousColumn,
     Column,
     CountStar,
     CrossJoin,
@@ -30,6 +30,7 @@ from sql_to_ibis.sql_objects import (
     GroupByColumn,
     Join,
     JoinBase,
+    Literal,
     Subquery,
     Table,
     Value,
@@ -61,6 +62,12 @@ TYPE_TO_PANDAS_TYPE = {
     "category": "category",
 }
 
+
+class TupleTree(Tree):
+    def __init__(self, data, children: Any, meta=None):
+        super().__init__(data, children, meta)
+
+
 for TYPE in PANDAS_TYPE_PYTHON_TYPE_FUNCTION:
     TYPE_TO_PANDAS_TYPE[TYPE] = TYPE
 
@@ -86,8 +93,7 @@ class SQLTransformer(TransformerBaseClass):
             column_name_map = {}
         if column_to_table_name is None:
             column_to_table_name = {}
-        TransformerBaseClass.__init__(
-            self,
+        super().__init__(
             table_name_map,
             table_map,
             column_name_map,
@@ -127,8 +133,7 @@ class SQLTransformer(TransformerBaseClass):
             raise TableExprDoesNotExist(table_name)
         true_name = self._table_name_map[table_name]
         if isinstance(alias, Tree) and alias.data == "alias_string":
-            alias_token: Token = alias.children[0]
-            alias = alias_token.value
+            alias = str(alias.children[0])
         table = Table(
             value=self._table_map[true_name].get_table_expr(),
             name=true_name,
@@ -146,7 +151,7 @@ class SQLTransformer(TransformerBaseClass):
         """
         order_type = rank_tree.data
         ascending = order_type == "order_asc"
-        return Token("order_by", (rank_tree.children[0].children, ascending))
+        return OrderByInfo(rank_tree.children[0].children[0], ascending)
 
     def integer(self, integer_token):
         """
@@ -163,7 +168,7 @@ class SQLTransformer(TransformerBaseClass):
         :param limit_count_value:
         :return:
         """
-        return Token("limit", limit_count_value)
+        return LimitExpression(limit_count_value)
 
     def query_expr(self, query_info: QueryInfo, *args):
         """
@@ -173,15 +178,23 @@ class SQLTransformer(TransformerBaseClass):
         :return: Query info
         """
         for token in args:
-            if isinstance(token, Token):
-                if token.type == "order_by":
-                    query_info.order_by.append(token.value)
-                elif token.type == "limit":
-                    query_info.limit = token.value
+            if isinstance(token, OrderByInfo):
+                query_info.add_order_by_info(token)
+            if isinstance(token, LimitExpression):
+                query_info.limit = token.limit
         return query_info
 
     def _handle_join_subqueries(self, join: JoinBase) -> QueryInfo:
-        info = QueryInfo(InternalTransformer.empty_transformer(),)
+        info = QueryInfo(
+            InternalTransformer(
+                join.get_tables(),
+                join.get_table_map(),
+                self._column_name_map,
+                self._column_to_table_name,
+                self._table_name_map,
+                self._alias_registry,
+            ),
+        )
         info.add_table(join)
         info.add_column(Column(name="*"))
         return info
@@ -193,8 +206,7 @@ class SQLTransformer(TransformerBaseClass):
         :param alias:
         :return:
         """
-        assert alias.data == "alias_string"
-        alias_name = alias.children[0].value
+        alias_name = str(alias.children[0])
         if isinstance(query_object, JoinBase):
             query_info = self._handle_join_subqueries(query_object)
         else:
@@ -210,7 +222,7 @@ class SQLTransformer(TransformerBaseClass):
 
     def column_name(self, *names):
         full_name = ".".join([str(name) for name in names])
-        return Tree("column_name", full_name)
+        return Tree("column_name", [full_name])
 
     def join(self, join_expression):
         """
@@ -220,7 +232,9 @@ class SQLTransformer(TransformerBaseClass):
         """
         return join_expression
 
-    def _determine_column_side(self, column, left_table: Table, right_table: Table):
+    def _determine_column_side(
+        self, column: str, left_table: Table, right_table: Table
+    ):
         """
         Check if column table prefix is one of the two tables (if there is one) AND
         the column has to be in one of the two tables
@@ -282,31 +296,11 @@ class SQLTransformer(TransformerBaseClass):
                     join_type = match.group("type")
             if join_type in {"full", "cross"}:
                 join_type = "outer"
-
-        # Check that there is a column from both sides
-        column_comparison = join_condition.children[0].children[0].children
-        column1 = str(column_comparison[0].children)
-        column2 = str(column_comparison[1].children)
-
-        column1_side, column1 = self._determine_column_side(column1, table1, table2)
-        column2_side, column2 = self._determine_column_side(column2, table1, table2)
-        if column1_side == column2_side:
-            raise Exception("Join columns must be one column from each join table!")
-        column1 = self._column_name_map[table1.name][column1]
-        column2 = self._column_name_map[table2.name][column2]
-        if column1_side == "left":
-            left_on = column1
-            right_on = column2
-        else:
-            left_on = column2
-            right_on = column1
-
         return Join(
             left_table=table1,
             right_table=table2,
             join_type=join_type,
-            left_on=left_on,
-            right_on=right_on,
+            join_condition=join_condition,
         )
 
     @staticmethod
@@ -356,7 +350,7 @@ class SQLTransformer(TransformerBaseClass):
                     where_expr = select_expression
 
         internal_transformer = InternalTransformer(
-            tables,
+            tables,  # type: ignore
             self._table_map,
             self._column_name_map,
             self._column_to_table_name,
@@ -364,19 +358,26 @@ class SQLTransformer(TransformerBaseClass):
             self._alias_registry,
         )
 
-        select_expressions_no_boolean_clauses = tuple(
-            select_expression
-            for select_expression in select_expressions
-            if isinstance(select_expression, Tree)
-            and select_expression.data not in ("having_expr", "where_expr")
-            or not isinstance(select_expression, Tree)
-        )
+        select_expressions_no_boolean_clauses: List[Union[str, Tree]] = []
+        distinct = False
+        for select_expression in select_expressions:
+            if isinstance(select_expression, Tree) and select_expression.data not in (
+                "having_expr",
+                "where_expr",
+            ):
+                select_expressions_no_boolean_clauses.append(select_expression)
+            if (
+                isinstance(select_expression, Token)
+                and select_expression.value.lower() == "distinct"
+            ):
+                distinct = True
 
         return QueryInfo(
             internal_transformer,
             select_expressions_no_boolean_clauses,
             having_expr=having_expr,
             where_expr=where_expr,
+            distinct=distinct,
         )
 
     def cross_join(self, table1: Table, table2: Table):
@@ -386,7 +387,10 @@ class SQLTransformer(TransformerBaseClass):
         :param table2: TableExpr2
         :return: Crossjoined dataframe
         """
-        return CrossJoin(left_table=table1, right_table=table2,)
+        return CrossJoin(
+            left_table=table1,
+            right_table=table2,
+        )
 
     def from_item(self, item):
         return item
@@ -505,16 +509,15 @@ class SQLTransformer(TransformerBaseClass):
         :param internal_transformer: Transformer to transform the where clauses
         :return: Filtered TableExpr
         """
-        where_value = None
         if where_expr is not None:
-            where_value_token = internal_transformer.transform(where_expr)
-            where_value = where_value_token.value
-        if where_value is not None:
-            return ibis_table.filter(where_value)
+            where_expression: WhereExpression = internal_transformer.transform(
+                where_expr
+            )
+            return ibis_table.filter(where_expression.value.get_value())
         return ibis_table
 
     def subquery_in(self, column: Tree, subquery: Subquery):
-        return Tree("subquery_in", (column, subquery))
+        return TupleTree("subquery_in", (column, subquery))
 
     def handle_selection(
         self, ibis_table: TableExpr, columns: List[Value]
@@ -529,22 +532,6 @@ class SQLTransformer(TransformerBaseClass):
             return ibis_table.projection(column_mutation)
         return ibis_table
 
-    def handle_duplicate_columns_in_join(
-        self, right_table: TableExpr, left_table: TableExpr, join: JoinBase
-    ):
-        duplicate_columns = set(left_table.columns).intersection(right_table.columns)
-        for column in duplicate_columns:
-            left_table = left_table.relabel({column: f"{join.left_table}." f"{column}"})
-            right_table = right_table.relabel(
-                {column: f"{join.right_table}" f".{column}"}
-            )
-
-        if isinstance(join, Join) and join.left_on == join.right_on:
-            join.left_on = f"{join.left_table}.{join.left_on}"
-            join.right_on = f"{join.right_table}.{join.right_on}"
-
-        return left_table, right_table
-
     @staticmethod
     def _columns_have_select_star(columns: List[Value]):
         for column in columns:
@@ -598,7 +585,12 @@ class SQLTransformer(TransformerBaseClass):
             all_columns += columns[table]
         return all_columns
 
-    def handle_join(self, join: JoinBase, columns: List[Value]) -> TableExpr:
+    def handle_join(
+        self,
+        join: JoinBase,
+        columns: List[Value],
+        internal_transformer: InternalTransformer,
+    ) -> TableExpr:
         """
         Return the table expr resulting from the join
         :param join:
@@ -618,10 +610,12 @@ class SQLTransformer(TransformerBaseClass):
         left_ibis_table = left_table.get_table_expr()
         right_ibis_table = right_table.get_table_expr()
         if isinstance(join, Join):
+            compiled_condition: Value = internal_transformer.transform(
+                join.join_condition
+            )
             result = left_ibis_table.join(
                 right_ibis_table,
-                predicates=left_ibis_table.get_column(join.left_on)
-                == right_ibis_table.get_column(join.right_on),
+                predicates=compiled_condition.get_value(),
                 how=join.join_type,
             )
         if isinstance(join, CrossJoin):
@@ -676,7 +670,11 @@ class SQLTransformer(TransformerBaseClass):
             raise Exception("No table specified")
         first_table = self.get_table_value(tables[0])
         if isinstance(first_table, JoinBase):
-            first_table = self.handle_join(join=first_table, columns=query_info.columns)
+            first_table = self.handle_join(
+                join=first_table,
+                columns=query_info.columns,
+                internal_transformer=query_info.internal_transformer,
+            )
         for table in tables[1:]:
             next_table = self.get_table_value(table)
             first_table = first_table.cross_join(next_table)
@@ -695,7 +693,6 @@ class SQLTransformer(TransformerBaseClass):
         query_info.perform_transformation()
 
         relation = self._get_relation(query_info)
-        print(relation)
 
         self._set_casing_for_groupby_names(query_info)
 
@@ -757,7 +754,9 @@ class SQLTransformer(TransformerBaseClass):
         return self._to_ibis_table(query_info)
 
     def union_all(
-        self, expr1: TableExpr, expr2: TableExpr,
+        self,
+        expr1: TableExpr,
+        expr2: TableExpr,
     ):
         """
         Return union distinct of two TableExpr
@@ -768,7 +767,9 @@ class SQLTransformer(TransformerBaseClass):
         return expr1.union(expr2)
 
     def union_distinct(
-        self, expr1: TableExpr, expr2: TableExpr,
+        self,
+        expr1: TableExpr,
+        expr2: TableExpr,
     ):
         """
         Return union distinct of two TableExpr
@@ -812,4 +813,5 @@ class SQLTransformer(TransformerBaseClass):
         :return:
         """
         DerivedColumn.reset_expression_count()
+        Literal.reset_literal_count()
         return table
